
#include <libgnomeuimm/appbar.h>
#include <libgnomeuimm/private/appbar_p.h>

// -*- C++ -*-
/* $Id: appbar.ccg,v 1.5 2001/12/08 14:19:04 daniel Exp $ */

/* appbar.ccg
 *
 * Copyright (C) 1998 EMC Capital Management Inc.
 * Developed by Havoc Pennington <hp@pobox.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */


namespace
{

const Glib::SignalProxyInfo AppBar_signal_clear_prompt_info =
{
  "clear_prompt",
  (GCallback) &Glib::SignalProxyNormal::slot0_void_callback,
  (GCallback) &Glib::SignalProxyNormal::slot0_void_callback
};


const Glib::SignalProxyInfo AppBar_signal_user_response_info =
{
  "user_response",
  (GCallback) &Glib::SignalProxyNormal::slot0_void_callback,
  (GCallback) &Glib::SignalProxyNormal::slot0_void_callback
};

} // anonymous namespace


namespace Glib
{

Gnome::UI::AppBar* wrap(GnomeAppBar* object, bool take_copy /* = false */)
{
  return dynamic_cast<Gnome::UI::AppBar *> (Glib::wrap_auto ((GObject*)(object), take_copy));
}

} /* namespace Glib */

namespace Gnome
{

namespace UI
{


/* The *_Class implementation: */

GType AppBar_Class::get_type()
{
  if(!gtype_) // create the GType if necessary
  {
    // TODO: This is currently just optimized away, apparently with no harm.
    // Is it actually needed?
    // Make sure that the parent type has been created.
    CppClassParent::CppObjectType::get_type();

    // Create the wrapper type, with the same class/instance size as the base type.
    register_derived_type(gnome_appbar_get_type(), (GClassInitFunc) &class_init_function);

    // Add derived versions of interfaces, if the C type implements any interfaces:
  }

  return gtype_;
}

void AppBar_Class::class_init_function(BaseClassType* klass)
{
  CppClassParent::class_init_function((CppClassParent::BaseClassType*) klass);
  klass->clear_prompt = &clear_prompt_callback;
  klass->user_response = &user_response_callback;
}


void AppBar_Class::clear_prompt_callback(GnomeAppBar* self)
{
  CppObjectType *const obj = dynamic_cast<CppObjectType*>(
      Glib::ObjectBase::_get_current_wrapper((GObject*)self));

  if(obj)
  {
    try
    {
      obj->on_clear_prompt();
    }
    catch(...)
    {
      Glib::exception_handlers_invoke();
    }
  }
  else
  {
    BaseClassType *const base = static_cast<BaseClassType*>(
        g_type_class_peek_parent(G_OBJECT_GET_CLASS(self)) // Get the parent class of the object class.
);
    g_assert(base != 0);

    if(base->clear_prompt)
      (*base->clear_prompt)(self);
  }
}

void AppBar_Class::user_response_callback(GnomeAppBar* self)
{
  CppObjectType *const obj = dynamic_cast<CppObjectType*>(
      Glib::ObjectBase::_get_current_wrapper((GObject*)self));

  if(obj)
  {
    try
    {
      obj->on_user_response();
    }
    catch(...)
    {
      Glib::exception_handlers_invoke();
    }
  }
  else
  {
    BaseClassType *const base = static_cast<BaseClassType*>(
        g_type_class_peek_parent(G_OBJECT_GET_CLASS(self)) // Get the parent class of the object class.
);
    g_assert(base != 0);

    if(base->user_response)
      (*base->user_response)(self);
  }
}


Glib::ObjectBase* AppBar_Class::wrap_new(GObject* o)
{
  return manage(new AppBar((GnomeAppBar*)(o)));
}


/* The implementation: */

AppBar::AppBar(GnomeAppBar* castitem)
: Gtk::HBox((GtkHBox*)(castitem))
{}

AppBar::~AppBar()
{
  destroy_();
}

AppBar::CppClassType AppBar::appbar_class_; //Initialize static member.

GType AppBar::get_type()
{
  return appbar_class_.get_type();
}

GType AppBar::get_base_type()
{
  return gnome_appbar_get_type();
}

AppBar::AppBar(bool has_progress, bool has_status, GnomePreferencesType interactivity)
: Gtk::HBox((GtkHBox*)( g_object_new(get_type(), "has_progress", static_cast<int>(has_progress), "has_status", static_cast<int>(has_status), "interactivity", interactivity, (char*)0) ))
{}

Gtk::ProgressBar* AppBar::get_progress()
{
  return Glib::wrap(gnome_appbar_get_progress(gobj()));
}

Gtk::Widget* AppBar::get_status()
{
  return Glib::wrap(gnome_appbar_get_status(gobj()));
}

void AppBar::set_default(const Glib::ustring& default_status)
{
  gnome_appbar_set_default(gobj(), default_status.c_str());
}

void AppBar::set_status(const Glib::ustring& status)
{
  gnome_appbar_set_status(gobj(), status.c_str());
}

void AppBar::push(const Glib::ustring& status)
{
  gnome_appbar_push(gobj(), status.c_str());
}

void AppBar::pop()
{
  gnome_appbar_pop(gobj());
}

void AppBar::clear_stack()
{
  gnome_appbar_clear_stack(gobj());
}

void AppBar::set_progress_percentage(gfloat percentage)
{
  gnome_appbar_set_progress_percentage(gobj(), percentage);
}

void AppBar::refresh()
{
  gnome_appbar_refresh(gobj());
}

void AppBar::prompt(const Glib::ustring& message, bool modal)
{
  gnome_appbar_set_prompt(gobj(), message.c_str(), static_cast<int>(modal));
}

void AppBar::clear_prompt()
{
  gnome_appbar_clear_prompt(gobj());
}

Glib::ustring AppBar::get_response() const
{
  return Glib::convert_return_gchar_ptr_to_ustring(gnome_appbar_get_response(const_cast<GnomeAppBar*>(gobj())));
}


Glib::SignalProxy0<void> AppBar::signal_clear_prompt()
{
  return Glib::SignalProxy0<void>(this, &AppBar_signal_clear_prompt_info);
}

Glib::SignalProxy0<void> AppBar::signal_user_response()
{
  return Glib::SignalProxy0<void>(this, &AppBar_signal_user_response_info);
}


void Gnome::UI::AppBar::on_clear_prompt()
{
  BaseClassType *const base = static_cast<BaseClassType*>(
      g_type_class_peek_parent(G_OBJECT_GET_CLASS(gobject_)) // Get the parent class of the object class.
);
  g_assert(base != 0);

  if(base->clear_prompt)
    (*base->clear_prompt)(gobj());
}

void Gnome::UI::AppBar::on_user_response()
{
  BaseClassType *const base = static_cast<BaseClassType*>(
      g_type_class_peek_parent(G_OBJECT_GET_CLASS(gobject_)) // Get the parent class of the object class.
);
  g_assert(base != 0);

  if(base->user_response)
    (*base->user_response)(gobj());
}


} // namespace UI

} // namespace Gnome

