
#include <libgnomeuimm/dateedit.h>
#include <libgnomeuimm/private/dateedit_p.h>

/* $Id: dateedit.ccg,v 1.4 2001/12/04 23:04:49 murrayc Exp $ */


/* dateedit.ccg
 * 
 * Copyright (C) 1999 The Gtk-- Development Team
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */


namespace
{

const Glib::SignalProxyInfo DateEdit_signal_date_changed_info =
{
  "date_changed",
  (GCallback) &Glib::SignalProxyNormal::slot0_void_callback,
  (GCallback) &Glib::SignalProxyNormal::slot0_void_callback
};


const Glib::SignalProxyInfo DateEdit_signal_time_changed_info =
{
  "time_changed",
  (GCallback) &Glib::SignalProxyNormal::slot0_void_callback,
  (GCallback) &Glib::SignalProxyNormal::slot0_void_callback
};

} // anonymous namespace


namespace Glib
{

Gnome::UI::DateEdit* wrap(GnomeDateEdit* object, bool take_copy /* = false */)
{
  return dynamic_cast<Gnome::UI::DateEdit *> (Glib::wrap_auto ((GObject*)(object), take_copy));
}

} /* namespace Glib */

namespace Gnome
{

namespace UI
{


/* The *_Class implementation: */

GType DateEdit_Class::get_type()
{
  if(!gtype_) // create the GType if necessary
  {
    // TODO: This is currently just optimized away, apparently with no harm.
    // Is it actually needed?
    // Make sure that the parent type has been created.
    CppClassParent::CppObjectType::get_type();

    // Create the wrapper type, with the same class/instance size as the base type.
    register_derived_type(gnome_date_edit_get_type(), (GClassInitFunc) &class_init_function);

    // Add derived versions of interfaces, if the C type implements any interfaces:
  }

  return gtype_;
}

void DateEdit_Class::class_init_function(BaseClassType* klass)
{
  CppClassParent::class_init_function((CppClassParent::BaseClassType*) klass);
  klass->date_changed = &date_changed_callback;
  klass->time_changed = &time_changed_callback;
}


void DateEdit_Class::date_changed_callback(GnomeDateEdit* self)
{
  CppObjectType *const obj = dynamic_cast<CppObjectType*>(
      Glib::ObjectBase::_get_current_wrapper((GObject*)self));

  if(obj)
  {
    try
    {
      obj->on_date_changed();
    }
    catch(...)
    {
      Glib::exception_handlers_invoke();
    }
  }
  else
  {
    BaseClassType *const base = static_cast<BaseClassType*>(
        g_type_class_peek_parent(G_OBJECT_GET_CLASS(self)) // Get the parent class of the object class.
);
    g_assert(base != 0);

    if(base->date_changed)
      (*base->date_changed)(self);
  }
}

void DateEdit_Class::time_changed_callback(GnomeDateEdit* self)
{
  CppObjectType *const obj = dynamic_cast<CppObjectType*>(
      Glib::ObjectBase::_get_current_wrapper((GObject*)self));

  if(obj)
  {
    try
    {
      obj->on_time_changed();
    }
    catch(...)
    {
      Glib::exception_handlers_invoke();
    }
  }
  else
  {
    BaseClassType *const base = static_cast<BaseClassType*>(
        g_type_class_peek_parent(G_OBJECT_GET_CLASS(self)) // Get the parent class of the object class.
);
    g_assert(base != 0);

    if(base->time_changed)
      (*base->time_changed)(self);
  }
}


Glib::ObjectBase* DateEdit_Class::wrap_new(GObject* o)
{
  return manage(new DateEdit((GnomeDateEdit*)(o)));
}


/* The implementation: */

DateEdit::DateEdit(GnomeDateEdit* castitem)
: Gtk::HBox((GtkHBox*)(castitem))
{}

DateEdit::~DateEdit()
{
  destroy_();
}

DateEdit::CppClassType DateEdit::dateedit_class_; //Initialize static member.

GType DateEdit::get_type()
{
  return dateedit_class_.get_type();
}

GType DateEdit::get_base_type()
{
  return gnome_date_edit_get_type();
}

DateEdit::DateEdit(time_t the_time, bool show_time, bool use_24_fomat)
: Gtk::HBox((GtkHBox*)( g_object_new(get_type(), "the_time", the_time, "show_time", static_cast<int>(show_time), "use_24_fomat", static_cast<int>(use_24_fomat), (char*)0) ))
{}

DateEdit::DateEdit(time_t the_time, GnomeDateEditFlags p2)
: Gtk::HBox((GtkHBox*)( g_object_new(get_type(), "the_time", the_time, "p2", p2, (char*)0) ))
{}

void DateEdit::set_time(time_t the_time)
{
  gnome_date_edit_set_time(gobj(), the_time);
}

time_t DateEdit::get_time() const
{
  return gnome_date_edit_get_time(const_cast<GnomeDateEdit*>(gobj()));
}

void DateEdit::set_popup_range(int low_hour, int up_hour)
{
  gnome_date_edit_set_popup_range(gobj(), low_hour, up_hour);
}

void DateEdit::set_flags(GnomeDateEditFlags flags)
{
  gnome_date_edit_set_flags(gobj(), flags);
}

GnomeDateEditFlags DateEdit::get_flags() const
{
  return (GnomeDateEditFlags)(gnome_date_edit_get_flags(const_cast<GnomeDateEdit*>(gobj())));
}

time_t DateEdit::get_initial_time() const
{
  return gnome_date_edit_get_initial_time(const_cast<GnomeDateEdit*>(gobj()));
}


Glib::SignalProxy0<void> DateEdit::signal_date_changed()
{
  return Glib::SignalProxy0<void>(this, &DateEdit_signal_date_changed_info);
}

Glib::SignalProxy0<void> DateEdit::signal_time_changed()
{
  return Glib::SignalProxy0<void>(this, &DateEdit_signal_time_changed_info);
}


Glib::PropertyProxy<gulong> DateEdit::property_time()
{
  return Glib::PropertyProxy<gulong>(this, "time");
}

Glib::PropertyProxy<GnomeDateEditFlags> DateEdit::property_dateedit_flags()
{
  return Glib::PropertyProxy<GnomeDateEditFlags>(this, "dateedit_flags");
}

Glib::PropertyProxy<int> DateEdit::property_lower_hour()
{
  return Glib::PropertyProxy<int>(this, "lower_hour");
}

Glib::PropertyProxy<int> DateEdit::property_upper_hour()
{
  return Glib::PropertyProxy<int>(this, "upper_hour");
}

Glib::PropertyProxy<gulong> DateEdit::property_initial_time()
{
  return Glib::PropertyProxy<gulong>(this, "initial_time");
}


void Gnome::UI::DateEdit::on_date_changed()
{
  BaseClassType *const base = static_cast<BaseClassType*>(
      g_type_class_peek_parent(G_OBJECT_GET_CLASS(gobject_)) // Get the parent class of the object class.
);
  g_assert(base != 0);

  if(base->date_changed)
    (*base->date_changed)(gobj());
}

void Gnome::UI::DateEdit::on_time_changed()
{
  BaseClassType *const base = static_cast<BaseClassType*>(
      g_type_class_peek_parent(G_OBJECT_GET_CLASS(gobject_)) // Get the parent class of the object class.
);
  g_assert(base != 0);

  if(base->time_changed)
    (*base->time_changed)(gobj());
}


} // namespace UI

} // namespace Gnome

