
#include <libgnomeuimm/icon-textitem.h>
#include <libgnomeuimm/private/icon-textitem_p.h>

/* $Id: icon-textitem.ccg,v 1.1 2002/02/20 19:22:22 murrayc Exp $ */

/* icon-textitem.ccg
 *
 * Copyright (C) 1999-2002 The libgnomeuimm Development Team
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <libgnomeui/gnome-icon-item.h>

namespace Gnome
{

namespace UI
{

IconTextItem::IconTextItem(Gnome::Canvas::Group& parentx)
: Gnome::Canvas::Item((GnomeCanvasItem*)( g_object_new( get_type(), 0 )))
{
  item_construct(parentx);
}


IconTextItem::IconTextItem(Gnome::Canvas::Group& parentx, int x, int y, int width, const Glib::ustring& fontname, const Glib::ustring& text, bool is_editable /* = true */)
: Gnome::Canvas::Item((GnomeCanvasItem*)( g_object_new( get_type(), 0 )))
{
  item_construct(parentx);
  gnome_icon_text_item_configure(gobj(), x, y, width, fontname.c_str(), text.c_str(), (gboolean)is_editable, false /* is_static */);
}

} /* namespace UI */
} /* namespace Gnome */


namespace
{


gboolean IconTextItem_signal_text_changed_callback(GnomeIconTextItem*, void* data)
{
  using namespace Gnome::UI;
  typedef SigC::Slot0<bool> SlotType;

  try
  {
    SigC::SlotNode *const slot = Glib::SignalProxyNormal::data_to_slot(data);
    return static_cast<int>(((*(SlotType::Proxy)(slot->proxy_))
        ( slot)));
  }
  catch(...)
  {
    Glib::exception_handlers_invoke();
  }

  typedef gboolean RType;
  return RType();
}

gboolean IconTextItem_signal_text_changed_notify_callback(GnomeIconTextItem*, void* data)
{
  using namespace Gnome::UI;
  typedef SigC::Slot0<void> SlotType;

  try
  {
    SigC::SlotNode *const slot = Glib::SignalProxyNormal::data_to_slot(data);
    (*(SlotType::Proxy)(slot->proxy_))
        ( slot);
  }
  catch(...)
  {
    Glib::exception_handlers_invoke();
  }

  typedef gboolean RType;
  return RType();
}


const Glib::SignalProxyInfo IconTextItem_signal_text_changed_info =
{
  "text_changed",
  (GCallback) &IconTextItem_signal_text_changed_callback,
  (GCallback) &IconTextItem_signal_text_changed_notify_callback
};


const Glib::SignalProxyInfo IconTextItem_signal_height_changed_info =
{
  "height_changed",
  (GCallback) &Glib::SignalProxyNormal::slot0_void_callback,
  (GCallback) &Glib::SignalProxyNormal::slot0_void_callback
};


const Glib::SignalProxyInfo IconTextItem_signal_width_changed_info =
{
  "width_changed",
  (GCallback) &Glib::SignalProxyNormal::slot0_void_callback,
  (GCallback) &Glib::SignalProxyNormal::slot0_void_callback
};


const Glib::SignalProxyInfo IconTextItem_signal_editing_started_info =
{
  "editing_started",
  (GCallback) &Glib::SignalProxyNormal::slot0_void_callback,
  (GCallback) &Glib::SignalProxyNormal::slot0_void_callback
};


const Glib::SignalProxyInfo IconTextItem_signal_editing_stopped_info =
{
  "editing_stopped",
  (GCallback) &Glib::SignalProxyNormal::slot0_void_callback,
  (GCallback) &Glib::SignalProxyNormal::slot0_void_callback
};


const Glib::SignalProxyInfo IconTextItem_signal_selection_started_info =
{
  "selection_started",
  (GCallback) &Glib::SignalProxyNormal::slot0_void_callback,
  (GCallback) &Glib::SignalProxyNormal::slot0_void_callback
};


const Glib::SignalProxyInfo IconTextItem_signal_selection_stopped_info =
{
  "selection_stopped",
  (GCallback) &Glib::SignalProxyNormal::slot0_void_callback,
  (GCallback) &Glib::SignalProxyNormal::slot0_void_callback
};

} // anonymous namespace


namespace Glib
{

Gnome::UI::IconTextItem* wrap(GnomeIconTextItem* object, bool take_copy /* = false */)
{
  return dynamic_cast<Gnome::UI::IconTextItem *> (Glib::wrap_auto ((GObject*)(object), take_copy));
}

} /* namespace Glib */

namespace Gnome
{

namespace UI
{


/* The *_Class implementation: */

GType IconTextItem_Class::get_type()
{
  if(!gtype_) // create the GType if necessary
  {
    // TODO: This is currently just optimized away, apparently with no harm.
    // Is it actually needed?
    // Make sure that the parent type has been created.
    CppClassParent::CppObjectType::get_type();

    // Create the wrapper type, with the same class/instance size as the base type.
    register_derived_type(gnome_icon_text_item_get_type(), (GClassInitFunc) &class_init_function);

    // Add derived versions of interfaces, if the C type implements any interfaces:
  }

  return gtype_;
}

void IconTextItem_Class::class_init_function(BaseClassType* klass)
{
  CppClassParent::class_init_function((CppClassParent::BaseClassType*) klass);
  klass->text_changed = &text_changed_callback;
  klass->height_changed = &height_changed_callback;
  klass->width_changed = &width_changed_callback;
  klass->editing_started = &editing_started_callback;
  klass->editing_stopped = &editing_stopped_callback;
  klass->selection_started = &selection_started_callback;
  klass->selection_stopped = &selection_stopped_callback;
}


gboolean IconTextItem_Class::text_changed_callback(GnomeIconTextItem* self)
{
  CppObjectType *const obj = dynamic_cast<CppObjectType*>(
      Glib::ObjectBase::_get_current_wrapper((GObject*)self));

  if(obj)
  {
    try
    {
      return static_cast<int>(obj->on_text_changed());
    }
    catch(...)
    {
      Glib::exception_handlers_invoke();
    }
  }
  else
  {
    BaseClassType *const base = static_cast<BaseClassType*>(
        g_type_class_peek_parent(G_OBJECT_GET_CLASS(self)) // Get the parent class of the object class.
);
    g_assert(base != 0);

    if(base->text_changed)
      (*base->text_changed)(self);
  }

  typedef gboolean RType;
  return RType();
}

void IconTextItem_Class::height_changed_callback(GnomeIconTextItem* self)
{
  CppObjectType *const obj = dynamic_cast<CppObjectType*>(
      Glib::ObjectBase::_get_current_wrapper((GObject*)self));

  if(obj)
  {
    try
    {
      obj->on_height_changed();
    }
    catch(...)
    {
      Glib::exception_handlers_invoke();
    }
  }
  else
  {
    BaseClassType *const base = static_cast<BaseClassType*>(
        g_type_class_peek_parent(G_OBJECT_GET_CLASS(self)) // Get the parent class of the object class.
);
    g_assert(base != 0);

    if(base->height_changed)
      (*base->height_changed)(self);
  }
}

void IconTextItem_Class::width_changed_callback(GnomeIconTextItem* self)
{
  CppObjectType *const obj = dynamic_cast<CppObjectType*>(
      Glib::ObjectBase::_get_current_wrapper((GObject*)self));

  if(obj)
  {
    try
    {
      obj->on_width_changed();
    }
    catch(...)
    {
      Glib::exception_handlers_invoke();
    }
  }
  else
  {
    BaseClassType *const base = static_cast<BaseClassType*>(
        g_type_class_peek_parent(G_OBJECT_GET_CLASS(self)) // Get the parent class of the object class.
);
    g_assert(base != 0);

    if(base->width_changed)
      (*base->width_changed)(self);
  }
}

void IconTextItem_Class::editing_started_callback(GnomeIconTextItem* self)
{
  CppObjectType *const obj = dynamic_cast<CppObjectType*>(
      Glib::ObjectBase::_get_current_wrapper((GObject*)self));

  if(obj)
  {
    try
    {
      obj->on_editing_started();
    }
    catch(...)
    {
      Glib::exception_handlers_invoke();
    }
  }
  else
  {
    BaseClassType *const base = static_cast<BaseClassType*>(
        g_type_class_peek_parent(G_OBJECT_GET_CLASS(self)) // Get the parent class of the object class.
);
    g_assert(base != 0);

    if(base->editing_started)
      (*base->editing_started)(self);
  }
}

void IconTextItem_Class::editing_stopped_callback(GnomeIconTextItem* self)
{
  CppObjectType *const obj = dynamic_cast<CppObjectType*>(
      Glib::ObjectBase::_get_current_wrapper((GObject*)self));

  if(obj)
  {
    try
    {
      obj->on_editing_stopped();
    }
    catch(...)
    {
      Glib::exception_handlers_invoke();
    }
  }
  else
  {
    BaseClassType *const base = static_cast<BaseClassType*>(
        g_type_class_peek_parent(G_OBJECT_GET_CLASS(self)) // Get the parent class of the object class.
);
    g_assert(base != 0);

    if(base->editing_stopped)
      (*base->editing_stopped)(self);
  }
}

void IconTextItem_Class::selection_started_callback(GnomeIconTextItem* self)
{
  CppObjectType *const obj = dynamic_cast<CppObjectType*>(
      Glib::ObjectBase::_get_current_wrapper((GObject*)self));

  if(obj)
  {
    try
    {
      obj->on_selection_started();
    }
    catch(...)
    {
      Glib::exception_handlers_invoke();
    }
  }
  else
  {
    BaseClassType *const base = static_cast<BaseClassType*>(
        g_type_class_peek_parent(G_OBJECT_GET_CLASS(self)) // Get the parent class of the object class.
);
    g_assert(base != 0);

    if(base->selection_started)
      (*base->selection_started)(self);
  }
}

void IconTextItem_Class::selection_stopped_callback(GnomeIconTextItem* self)
{
  CppObjectType *const obj = dynamic_cast<CppObjectType*>(
      Glib::ObjectBase::_get_current_wrapper((GObject*)self));

  if(obj)
  {
    try
    {
      obj->on_selection_stopped();
    }
    catch(...)
    {
      Glib::exception_handlers_invoke();
    }
  }
  else
  {
    BaseClassType *const base = static_cast<BaseClassType*>(
        g_type_class_peek_parent(G_OBJECT_GET_CLASS(self)) // Get the parent class of the object class.
);
    g_assert(base != 0);

    if(base->selection_stopped)
      (*base->selection_stopped)(self);
  }
}


Glib::ObjectBase* IconTextItem_Class::wrap_new(GObject* o)
{
  return manage(new IconTextItem((GnomeIconTextItem*)(o)));
}


/* The implementation: */

IconTextItem::IconTextItem(GnomeIconTextItem* castitem)
: Gnome::Canvas::Item((GnomeCanvasItem*)(castitem))
{}

IconTextItem::~IconTextItem()
{
  destroy_();
}

IconTextItem::CppClassType IconTextItem::icontextitem_class_; //Initialize static member.

GType IconTextItem::get_type()
{
  return icontextitem_class_.get_type();
}

GType IconTextItem::get_base_type()
{
  return gnome_icon_text_item_get_type();
}

void IconTextItem::set_xy(int x, int y)
{
  gnome_icon_text_item_setxy(gobj(), x, y);
}

void IconTextItem::select(bool select)
{
  gnome_icon_text_item_select(gobj(), static_cast<int>(select));
}

void IconTextItem::focus(bool focused)
{
  gnome_icon_text_item_focus(gobj(), static_cast<int>(focused));
}

Glib::ustring IconTextItem::get_text()
{
  return Glib::convert_const_gchar_ptr_to_ustring(gnome_icon_text_item_get_text(gobj()));
}

void IconTextItem::start_editing()
{
  gnome_icon_text_item_start_editing(gobj());
}

void IconTextItem::stop_editing(bool accept)
{
  gnome_icon_text_item_stop_editing(gobj(), static_cast<int>(accept));
}

Glib::RefPtr<Gtk::Editable> IconTextItem::get_editable()
{
  return Glib::wrap(gnome_icon_text_item_get_editable(gobj()));
}


Glib::SignalProxy0<bool> IconTextItem::signal_text_changed()
{
  return Glib::SignalProxy0<bool>(this, &IconTextItem_signal_text_changed_info);
}

Glib::SignalProxy0<void> IconTextItem::signal_height_changed()
{
  return Glib::SignalProxy0<void>(this, &IconTextItem_signal_height_changed_info);
}

Glib::SignalProxy0<void> IconTextItem::signal_width_changed()
{
  return Glib::SignalProxy0<void>(this, &IconTextItem_signal_width_changed_info);
}

Glib::SignalProxy0<void> IconTextItem::signal_editing_started()
{
  return Glib::SignalProxy0<void>(this, &IconTextItem_signal_editing_started_info);
}

Glib::SignalProxy0<void> IconTextItem::signal_editing_stopped()
{
  return Glib::SignalProxy0<void>(this, &IconTextItem_signal_editing_stopped_info);
}

Glib::SignalProxy0<void> IconTextItem::signal_selection_started()
{
  return Glib::SignalProxy0<void>(this, &IconTextItem_signal_selection_started_info);
}

Glib::SignalProxy0<void> IconTextItem::signal_selection_stopped()
{
  return Glib::SignalProxy0<void>(this, &IconTextItem_signal_selection_stopped_info);
}


bool Gnome::UI::IconTextItem::on_text_changed()
{
  BaseClassType *const base = static_cast<BaseClassType*>(
      g_type_class_peek_parent(G_OBJECT_GET_CLASS(gobject_)) // Get the parent class of the object class.
);
  g_assert(base != 0);

  if(base->text_changed)
    return (*base->text_changed)(gobj());

  typedef bool RType;
  return RType();
}

void Gnome::UI::IconTextItem::on_height_changed()
{
  BaseClassType *const base = static_cast<BaseClassType*>(
      g_type_class_peek_parent(G_OBJECT_GET_CLASS(gobject_)) // Get the parent class of the object class.
);
  g_assert(base != 0);

  if(base->height_changed)
    (*base->height_changed)(gobj());
}

void Gnome::UI::IconTextItem::on_width_changed()
{
  BaseClassType *const base = static_cast<BaseClassType*>(
      g_type_class_peek_parent(G_OBJECT_GET_CLASS(gobject_)) // Get the parent class of the object class.
);
  g_assert(base != 0);

  if(base->width_changed)
    (*base->width_changed)(gobj());
}

void Gnome::UI::IconTextItem::on_editing_started()
{
  BaseClassType *const base = static_cast<BaseClassType*>(
      g_type_class_peek_parent(G_OBJECT_GET_CLASS(gobject_)) // Get the parent class of the object class.
);
  g_assert(base != 0);

  if(base->editing_started)
    (*base->editing_started)(gobj());
}

void Gnome::UI::IconTextItem::on_editing_stopped()
{
  BaseClassType *const base = static_cast<BaseClassType*>(
      g_type_class_peek_parent(G_OBJECT_GET_CLASS(gobject_)) // Get the parent class of the object class.
);
  g_assert(base != 0);

  if(base->editing_stopped)
    (*base->editing_stopped)(gobj());
}

void Gnome::UI::IconTextItem::on_selection_started()
{
  BaseClassType *const base = static_cast<BaseClassType*>(
      g_type_class_peek_parent(G_OBJECT_GET_CLASS(gobject_)) // Get the parent class of the object class.
);
  g_assert(base != 0);

  if(base->selection_started)
    (*base->selection_started)(gobj());
}

void Gnome::UI::IconTextItem::on_selection_stopped()
{
  BaseClassType *const base = static_cast<BaseClassType*>(
      g_type_class_peek_parent(G_OBJECT_GET_CLASS(gobject_)) // Get the parent class of the object class.
);
  g_assert(base != 0);

  if(base->selection_stopped)
    (*base->selection_stopped)(gobj());
}


} // namespace UI

} // namespace Gnome

